@@ -314,16 +314,12 @@ class Limits:
 
 
 class Proxy:
-    def __init__(
-        self, url: URLTypes, *, headers: HeaderTypes = None, mode: str = "DEFAULT"
-    ):
+    def __init__(self, url: URLTypes, *, headers: HeaderTypes = None):
         url = URL(url)
         headers = Headers(headers)
 
         if url.scheme not in ("http", "https"):
             raise ValueError(f"Unknown scheme for proxy URL {url!r}")
-        if mode not in ("DEFAULT", "FORWARD_ONLY", "TUNNEL_ONLY"):
-            raise ValueError(f"Unknown proxy mode {mode!r}")
 
         if url.username or url.password:
             headers.setdefault(
@@ -336,7 +332,6 @@ class Proxy:
 
         self.url = url
         self.headers = headers
-        self.mode = mode
 
     def _build_auth_header(self, username: str, password: str) -> str:
         userpass = (username.encode("utf-8"), password.encode("utf-8"))
@@ -344,11 +339,7 @@ class Proxy:
         return f"Basic {token}"
 
     def __repr__(self) -> str:
-        return (
-            f"Proxy(url={str(self.url)!r}, "
-            f"headers={dict(self.headers)!r}, "
-            f"mode={self.mode!r})"
-        )
+        return f"Proxy(url={str(self.url)!r}, headers={dict(self.headers)!r})"
 
 
 DEFAULT_TIMEOUT_CONFIG = Timeout(timeout=5.0)
